#  Copyright (c) Microsoft Corporation. All rights reserved.
#  Licensed under the MIT License.
from __future__ import annotations

import json

from agents import Agent, HandoffInputData, Runner, function_tool, handoff
from agents.extensions import handoff_filters
from agents.models import is_gpt_5_default


def spanish_handoff_message_filter(handoff_message_data: HandoffInputData) -> HandoffInputData:
    if is_gpt_5_default():
        print("gpt-5 is enabled, so we're not filtering the input history")
        # when using gpt-5, removing some of the items could break things, so we do this filtering only for other models
        return HandoffInputData(
            input_history=handoff_message_data.input_history,
            pre_handoff_items=tuple(handoff_message_data.pre_handoff_items),
            new_items=tuple(handoff_message_data.new_items),
        )

    # First, we'll remove any tool-related messages from the message history
    handoff_message_data = handoff_filters.remove_all_tools(handoff_message_data)

    # Second, we'll also remove the first two items from the history, just for demonstration
    history = (
        tuple(handoff_message_data.input_history[2:])
        if isinstance(handoff_message_data.input_history, tuple)
        else handoff_message_data.input_history
    )

    # or, you can use the HandoffInputData.clone(kwargs) method
    return HandoffInputData(
        input_history=history,
        pre_handoff_items=tuple(handoff_message_data.pre_handoff_items),
        new_items=tuple(handoff_message_data.new_items),
    )


def main(random_number_tool):
    first_agent = Agent(
        name="Assistant",
        instructions="Be extremely concise.",
        tools=[random_number_tool],
    )

    spanish_agent = Agent(
        name="Spanish Assistant",
        instructions="You only speak Spanish and are extremely concise.",
        handoff_description="A Spanish-speaking assistant.",
    )

    second_agent = Agent(
        name="Assistant",
        instructions=(
            "Be a helpful assistant. If the user speaks Spanish, handoff to the Spanish assistant."
        ),
        handoffs=[handoff(spanish_agent, input_filter=spanish_handoff_message_filter)],
    )

    # 1. Send a regular message to the first agent
    result = Runner.run_sync(first_agent, input="Hi, my name is Sora.")

    print("Step 1 done")

    # 2. Ask it to generate a number
    result = Runner.run_sync(
        first_agent,
        input=result.to_input_list()
        + [{"content": "Can you generate a random number between 0 and 100?", "role": "user"}],
    )

    print("Step 2 done")

    # 3. Call the second agent
    result = Runner.run_sync(
        second_agent,
        input=result.to_input_list()
        + [
            {
                "content": "I live in New York City. Whats the population of the city?",
                "role": "user",
            }
        ],
    )

    print("Step 3 done")

    # 4. Cause a handoff to occur
    result = Runner.run_sync(
        second_agent,
        input=result.to_input_list()
        + [
            {
                "content": "Por favor habla en español. ¿Cuál es mi nombre y dónde vivo?",
                "role": "user",
            }
        ],
    )

    print("Step 4 done")

    print("\n===Final messages===\n")

    # 5. That should have caused spanish_handoff_message_filter to be called, which means the
    # output should be missing the first two messages, and have no tool calls.
    # Let's print the messages to see what happened
    for message in result.to_input_list():
        print(json.dumps(message, indent=2))
        # tool_calls = message.tool_calls if isinstance(message, AssistantMessage) else None

        # print(f"{message.role}: {message.content}\n  - Tool calls: {tool_calls or 'None'}")
        """
        $python examples/handoffs/message_filter.py
        Step 1 done
        Step 2 done
        Step 3 done
        Step 4 done

        ===Final messages===

        {
            "content": "Can you generate a random number between 0 and 100?",
            "role": "user"
        }
        {
        "id": "...",
        "content": [
            {
            "annotations": [],
            "text": "Sure! Here's a random number between 0 and 100: **42**.",
            "type": "output_text"
            }
        ],
        "role": "assistant",
        "status": "completed",
        "type": "message"
        }
        {
        "content": "I live in New York City. Whats the population of the city?",
        "role": "user"
        }
        {
        "id": "...",
        "content": [
            {
            "annotations": [],
            "text": "As of the most recent estimates, the population of New York City is approximately 8.6 million people. However, this number is constantly changing due to various factors such as migration and birth rates. For the latest and most accurate information, it's always a good idea to check the official data from sources like the U.S. Census Bureau.",
            "type": "output_text"
            }
        ],
        "role": "assistant",
        "status": "completed",
        "type": "message"
        }
        {
        "content": "Por favor habla en espa\u00f1ol. \u00bfCu\u00e1l es mi nombre y d\u00f3nde vivo?",
        "role": "user"
        }
        {
        "id": "...",
        "content": [
            {
            "annotations": [],
            "text": "No tengo acceso a esa informaci\u00f3n personal, solo s\u00e9 lo que me has contado: vives en Nueva York.",
            "type": "output_text"
            }
        ],
        "role": "assistant",
        "status": "completed",
        "type": "message"
        }
        """

    return result.final_output
